c=======================================================================
c***********************************************************************
      subroutine nonlineq(xa,xb,nint,x,fx,ermax,nitmax,imeth,num)
c***********************************************************************

c=======================================================================
c   Subroutine nonlineq solves a non linear equation of the type:
c           f(x) = 0
c   by the bisection method or the "regula falsi" method.
c   It calls the user-defined subroutine "funcx(x,fx)" which
c   defines the f(x) function. 
c
c      xa = lower bound of interval in which the solution is 
c           searched for
c      xb = upper bound of interval in which the solution is 
c           searched for
c      nint = number of intervals into which [xa,xb] is divided
c           when searching for a (smaller) interval of changing
c           sign (e.g. 10)
c      x = - at entry, x is set to any value between a and b;
c            it may represent an "initial guess" of the solution
c          - at exit, x contains the estimated value of the zero 
c            of the function 
c      fx = at output, fx contains f(x), i.e. fx should be close to
c           0, if x contains a good evaluation of the zero
c      ermax = maximum acceptable error on fx, 
c               i.e., when (abs(fx).le.ermax) the zero is accepted
c      nitmax = maximum number of iterations allowed (e.g., 100)
c      imeth = flag to select resolution method. Set imeth to 0 for
c              bisection method and to 1 for "regula falsi"
c      num = at exit, contains total number of iterations performed
c   
c   The variable "precmc" below must be set to a small number
c   representing machine precision. The relative error on the
c   zero (x) cannot be smaller than precmc. With single precision
c   a value of precmc of 1.e-6 is recommended. For double precision
c   a value as small as 1.d-12 can be used.
c=======================================================================
c      implicit double precision (a-h,o-z)
      implicit none
      real*4 x,fx
c-----------------------------------------------
c JLP ajouté pour implicit none
c
      integer imeth,k,nint,nit,nitmax,num
      real*8  delx,ermax,err,errx,fx1,fx2,precmc,prod,prod1,residu,x_old
     &      ,x1,x2,xa,xb
c
c-----------------------------------------------

      precmc = 1.d-10

      num = 0

      delx = (xb-xa)/float(nint)

c=======================================================================
c   initial try with previous value of x and values smaller or 
c   larger by an amount delx 
c   (sets values for x1 and x2 with fx1*fx2 < 0)
c=======================================================================

      call funcx(x, fx)

      err = abs(fx)
      if (err.le.ermax) goto 300

      x1 = x 
      fx1 = fx

      x = x1+delx
      if (x.gt.xb) x=xb
      if (x.lt.xa) x=xa

      num = num + 1
      call funcx(x, fx)

      prod = fx*fx1
      if (prod.gt.0) then
         x = x1-delx
         if (x.gt.xb) x=xb
         if (x.lt.xa) x=xa
         call funcx(x, fx)

         prod1 = fx*fx1
         if (prod1.le.0) then
            x2 = x
            fx2 = fx
            goto 200
         endif
      else
         x2 = x
         fx2 = fx
         goto 200
      endif

c=======================================================================
c   if initial try fails, search for the solution over the  
c   entire range xa to xb 
c=======================================================================

      x = xa

      num = num + 1
      call funcx(x, fx)

      x1 = x
      fx1 = fx

      do k = 1, nint
         x = xa+delx*float(k)
         num = num + 1

         call funcx(x, fx)

         prod = fx1*fx
         if (prod.gt.0.) then
            x1 = x 
            fx1 = fx 
         else
            x2 = x
            fx2 = fx
            goto 200
         endif 

      enddo

      write(*,*)'No solution found for x between xa and xb'
      write(*,*)' x=',x,' fx=',fx
      write(*,*)'program stop'
      stop

c=======================================================================
c   after finding two values of x with fx of different signs,
c   we solve the equation by bisection (imeth=0) or "regula falsi" 
c   (imeth=1) method  
c=======================================================================

 200  continue

c      write(*,*)'Interval of changing sign found'

      do nit = 1, nitmax

         num = num + 1

         x_old = x

         if (imeth.eq.0) then
            x = (x1+x2)/2.
         else
            residu = -fx1*(x2-x1)/(fx2-fx1)
            x = x1 + residu
         endif

         call funcx(x, fx)

         err=abs(fx)

         if(x.ne.0.) then 
            errx = abs((x-x_old)/x)
         else
            errx = abs(x-x_old)
         endif
         if ((err.le.ermax).or.(errx.le.precmc)) goto 300

         prod = fx*fx2
         if (prod.gt.0) then
           x2 = x
           fx2 = fx
         else
           x1 = x
           fx1 = fx
         endif

      enddo

      write(*,*)'Too much iterations in subroutine nonlineq'
      write(*,*)'num=',num,' x=',x
      write(*,*)'fx=',fx
      write(*,*)'program stop'
      stop

 300  continue


      return
      end subroutine nonlineq