!   +------------------------------------------------------------------+
!   |  Subroutine CAR2MAR                      03/04/2024   INTERPOL   |
!   +------------------------------------------------------------------+
!   |                                                                  |
!   | This routine is a linear interpolation of a 2D scalar fields     |
!   | from a non-regular grid to a non-regular grid. It is designed to |
!   | interpol CARAIB data on MAR grid.                                |
!   |                                                                  |
!   |                                                                  |
!   | Input : caraib_lon(cx, cy) : Input CARIB points position x(i,j)  |
!   | ^^^^^^^ caraib_lat(cx, cy) :   "     "    "       "     y(i,j)   |
!   |         caraib_var(cx, cy) : Input Caraib field values           |
!   |         mar_lon(mx, my)   : Output MAR positions x(i,j)          |
!   |         mar_lat(mx, my)   : Output MAR positions y(i,j)          |
!   |         mar_var(mx, my)   : Output field values                  |
!   |                                                                  |
!   | Output: mar_var(mx, my) : Output field values                    |
!   | ^^^^^^^                                                          |
!   |                                                                  |
!   | T. Dethinne remark (03/04/2024): this routine is juste a routine |
!   | from NESTOR intMAR) but adapted for the coupling of MAR - CARAIB |
!   |                                                                  |
!   +------------------------------------------------------------------+

      SUBROUTINE car2mar(caraib_lon,caraib_lat,caraib_var, mar_lon,mar_lat,mar_var)
      
      use mardim
      IMPLICIT NONE

! +---General and local variables
! +   ---------------------------

      INTEGER  i,j,k,l,k1,k2,l1,l2,ii,jj

      INTEGER  c_ii_min(mx,my), c_ii_max(mx,my) 
      INTEGER  c_jj_min(mx,my), c_jj_max(mx,my)  

      REAL caraib_lon(cx,cy), caraib_lat(cx,cy), caraib_var(cx,cy), &
           mar_lon(mx,my), mar_lat(mx,my), mar_var(mx,my), &
           int_O(mx,my), epsi, xx, dist, dist_O, &
           c_delta_lat(mx,my), c_delta_lon(mx,my)

      DATA epsi / 0.1 /

      common/c_ii_min/c_ii_max,c_jj_min,c_jj_max
      common/c_delta_lat/c_delta_lon

! +   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

! +---Computes neigbors
! +   -----------------
      !if (iterun.lt.1) then 

       c_ii_min = cx ; c_ii_max = 1
       c_jj_min = cy ; c_jj_max = 1
      
       DO k=1,mx 
       DO l=1,my 


        k1=max(1,min(mx,k+1))
        k2=max(1,min(mx,k-1))

        l1=max(1,min(my,l+1))
        l2=max(1,min(my,l-1))
       
        c_delta_lat(k,l)=max(abs(mar_lat(k,l1)-mar_lat(k,l)), &
                             abs(mar_lat(k,l2)-mar_lat(k,l)), &
                             abs(mar_lat(k1,l)-mar_lat(k,l)), &
                             abs(mar_lat(k2,l)-mar_lat(k,l)))

        c_delta_lon(k,l)=max(abs(mar_lon(k,l1)-mar_lon(k,l)), &
                             abs(mar_lon(k,l2)-mar_lon(k,l)), &
                             abs(mar_lon(k1,l)-mar_lon(k,l)), &
                             abs(mar_lon(k2,l)-mar_lon(k,l)))
       

        int_O(k,l)=0.
       
        xx=0.05

        do while (int_O(k,l).le.3.)

         c_delta_lat(k,l)=c_delta_lat(k,l)*(1+xx)
         c_delta_lon(k,l)=c_delta_lon(k,l)*(1+xx)
         xx=xx+0.05
       
         do i = 1, cx 
         do j = 1, cy
          IF(abs(mar_lon(k,l)-caraib_lon(i,j)).le.c_delta_lon(k,l).and. & 
             abs(mar_lat(k,l)-caraib_lat(i,j)).le.c_delta_lat(k,l))then
           int_O(k,l) = int_O(k,l)+1
           c_jj_min(k,l) = min(j,c_jj_min(k,l))
           c_ii_min(k,l) = min(i,c_ii_min(k,l))  
           c_jj_max(k,l) = max(j,c_jj_max(k,l))
           c_ii_max(k,l) = max(i,c_ii_max(k,l)) 
          ENDIF
         end do
         end do

         if(xx.ge.5) then
         WRITE(6,*) 'No cell of input grid includes an output grid'
         WRITE(6,*) 'point.                     --- STOP in INTnrg.'
         stop
         endif

        enddo

       ENDDO
       ENDDO

       !endif    ! if first time step      

! +   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      DO k=1,mx 
      DO l=1,my 

       int_O(k,l)=0.
       mar_var(k,l)=0.
        
       do i = c_ii_min(k,l),c_ii_max(k,l)
       do j = c_jj_min(k,l),c_jj_max(k,l)  
        IF(abs(mar_lon(k,l)-caraib_lon(i,j)).le.c_delta_lon(k,l).and.  &
           abs(mar_lat(k,l)-caraib_lat(i,j)).le.c_delta_lat(k,l))then

          dist_O      = dist(mar_lon(k,l),mar_lat(k,l), caraib_lon(i,j),caraib_lat(i,j))

           int_O(k,l) = int_O(k,l)+           1./max(1.,dist_O)
           mar_var(k,l) = mar_var(k,l)+caraib_var(i,j)*1./max(1.,dist_O)
         ENDIF
       
       end do
       end do
  
       mar_var(k,l)=mar_var(k,l)/int_O(k,l)
     
      ENDDO
      ENDDO

      RETURN
      END

!   +------------------------------------------------------------------+
!   |  Function dist                             31/08/2004   NESTING  |
!   +------------------------------------------------------------------+
!   |                                                                  |      
!   | J.-F. Grailet remark (09/05/2022): this function was originally  |
!   | not documented, but given that it was declared in the original   |
!   | INTnrg2.f source file (in practical equivalent to this file), it |
!   | may have been written roughly around the same time or before     |
!   | (2004). The name and purpose are self-explanatory.               |
!   |                                                                  |
!   |                                                                  |
!   +------------------------------------------------------------------+

      function dist(lon2o,lat2o,lon1o,lat1o)

      implicit none
      real,parameter :: pi   = 3.141592
      real,parameter :: R    = 6371.

      real :: lon1,lat1
      real :: lon2,lat2
      real :: dlat,dlon,a,c,dist
      real :: lon2o,lat2o,lon1o,lat1o

      lon1=lon1o * Pi/180.
      lon2=lon2o * Pi/180.
      lat1=lat1o * Pi/180.
      lat2=lat2o * Pi/180.

      dlat = (lat2-lat1)
      dlon = (lon2-lon1)
         a =  sin(dLat/2.) * sin(dLat/2.) + cos(lat1) * cos(lat2) * sin(dLon/2.) * sin(dLon/2.)
         c =  2. * atan2(sqrt(a), sqrt(1.-a))
      dist =  R * c

      end function
