
subroutine DYNfil_3D(f3_fil, e3_fil, k3_fil)
    ! +------------------------------------------------------------------------+
    ! | MAR DYNAMICS FILTER (3-D)                           19-11-2012-XF  MAR |
    ! |   subroutine DYNfil_3D  is used to Filter Horizontal Fields in 3D Code |
    ! |                                                                        |
    ! +------------------------------------------------------------------------+
    ! |                                                                        |
    ! |   INPUT:   f3_fil(i,j,k): variable to be filtered (in surface k)       |
    ! |   ^^^^^    e3_fil(    k): value of the selectivity parameter           |
    ! |            k3_fil       : vertical dimension of the variable           |
    ! |                                                                        |
    ! |   OUTPUT:  f3_fil(i,j,k)                                               |
    ! |   ^^^^^^^                                                              |
    ! |                                                                        |
    ! |   LATERAL BOUNDARIES:                                                  |
    ! |   ^^^^^^^^^^^^^^^^^^^                                                  |
    ! |   1. The value    of the variable is fixed at the Boundary             |
    ! |   2. Filter Selectivity Parameter may be increased near the Boundary   |
    ! |                                                                (#EP)   |
    ! |                                                                        |
    ! |   REFER. : Raymond and Garder, MWR 116, Jan 1988, p209                 |
    ! |   ^^^^^^^^                                                             |
    ! +------------------------------------------------------------------------+

    use marphy
    use mardim
    use margrd
    use mar_wk

    implicit none

    integer i, j, k, m
    real f3_fil(mx, my, mz), e3_fil(mz)
    integer k3_fil

    ! +--Local  Variables
    ! +  ================

    real eps2(mz)
    real eps3(mz)
    real eps4(mz)
    real eps5(mz)
    real a1_fil(1:mx, mz), b1_fil(1:mx, mz), aa_fil(mz)
    real a2_fil(1:my, mz), b2_fil(1:my, mz), bb_fil(mz)

    ! +--Initialisation
    ! +  ==============

    m = mx
    m1 = mx1
    m2 = mx2

    mn = my
    mn1 = my1
    mn2 = my2

    ! +--1st Matrix Initialisation
    ! +  -------------------------

    do k = 1, k3_fil
        a1_fil(1, k) = 0.0
        b1_fil(1, k) = 1.0
        a1_fil(mx, k) = 0.0
        b1_fil(mx, k) = 1.0

        aa_fil(k) = 1.0 - e3_fil(k)
        bb_fil(k) = 2.0 * (1.0 + e3_fil(k))

        do i = ip11, mx1
            a1_fil(i, k) = aa_fil(k)
            b1_fil(i, k) = bb_fil(k)
        enddo

        ! +--2th Matrix Initialisation
        ! +  -------------------------

        a2_fil(1, k) = 0.0
        b2_fil(1, k) = 1.0
        a2_fil(my, k) = 0.0
        b2_fil(my, k) = 1.0

        do j = jp11, my1
            a2_fil(j, k) = aa_fil(k)
            b2_fil(j, k) = bb_fil(k)
        enddo

    enddo

    ! +--1st Equations System
    ! +  ====================

    ! +--Gaussian Elimination Algorithm: Set Up
    ! +  --------------------------------------

    do k = 1, k3_fil
        do j = jp11, my1

            WTxyz4(1, j, k) = &
                f3_fil(1, jm1(j), k) + 2.0 * f3_fil(1, j, k) + &
                f3_fil(1, jp1(j), k)
            WTxyz4(mx, j, k) = &
                f3_fil(mx, jm1(j), k) + 2.0 * f3_fil(mx, j, k) + &
                f3_fil(mx, jp1(j), k)

            do i = ip11, mx1
                WTxyz4(i, j, k) = &
                    f3_fil(im1(i), jp1(j), k) + 2.0 * f3_fil(i, jp1(j), k) &
                    + f3_fil(ip1(i), jp1(j), k) + &
                    2.0 * f3_fil(im1(i), j, k) + 4.0 * &
                    f3_fil(i, j, k) &
                    + 2.0 * f3_fil(ip1(i), j, k) + &
                    f3_fil(im1(i), jm1(j), k) + 2.0 * &
                    f3_fil(i, jm1(j), k) &
                    + f3_fil(ip1(i), jm1(j), k)
            enddo

        enddo
    enddo

    ! +--Gaussian Elimination Algorithm: F-B Sweep ==> WTxyz7
    ! +  ----------------------------------------------------
    ! +
    do k = 1, k3_fil
        do j = 1, my

            ! +--Forward  Sweep

            WTxyz6(1, j, k) = -a1_fil(1, k) / b1_fil(1, k)
            WTxyz7(1, j, k) = WTxyz4(1, j, k) / b1_fil(1, k)
            do i = ip11, mx
                WTxyz6(i, j, k) = -a1_fil(i, k) &
                                  / (a1_fil(i, k) * WTxyz6(i - 1, j, k) + b1_fil(i, k))
                WTxyz7(i, j, k) = (WTxyz4(i, j, k) - a1_fil(i, k) * &
                                   WTxyz7(i - 1, j, k)) &
                                  / (a1_fil(i, k) * WTxyz6(i - 1, j, k) + b1_fil(i, k))
            enddo

            ! +--Backward Sweep

            do i = mx1, 1, -1
                WTxyz7(i, j, k) = WTxyz6(i, j, k) * WTxyz7(i + 1, j, k) + &
                                  WTxyz7(i, j, k)
            enddo

        enddo
    enddo

    ! +--2th Equations System
    ! +  ====================

    ! +--Gaussian Elimination Algorithm: Set Up
    ! +  --------------------------------------

    do k = 1, k3_fil
        do i = ip11, mx1
            WTxyz4(i, 1, k) = f3_fil(i, 1, k)
            WTxyz4(i, my, k) = f3_fil(i, my, k)
            do j = jp11, my1
                WTxyz4(i, j, k) = WTxyz7(i, j, k)
            enddo
        enddo
    enddo

    ! +--Gaussian Elimination Algorithm: F-B Sweep ==> WTxyz7
    ! +  ----------------------------------------------------

    do k = 1, k3_fil
        do i = 1, mx

            ! +--Forward  Sweep

            WTxyz6(i, 1, k) = -a2_fil(1, k) / b2_fil(1, k)
            WTxyz7(i, 1, k) = WTxyz4(i, 1, k) / b2_fil(1, k)
            do j = jp11, my
                WTxyz6(i, j, k) = -a2_fil(j, k) &
                                  / (a2_fil(j, k) * WTxyz6(i, j - 1, k) + b2_fil(j, k))
                WTxyz7(i, j, k) = (WTxyz4(i, j, k) - a2_fil(j, k) * &
                                   WTxyz7(i, j - 1, k)) &
                                  / (a2_fil(j, k) * WTxyz6(i, j - 1, k) + b2_fil(j, k))
            enddo

            ! +--Backward Sweep

            do j = my1, 1, -1
                WTxyz7(i, j, k) = WTxyz6(i, j, k) * WTxyz7(i, j + 1, k) + &
                                  WTxyz7(i, j, k)
            enddo

        enddo
    enddo

    ! +--Result
    ! +  ======

    do k = 1, k3_fil
        do j = jp11, my1
            do i = ip11, mx1
                f3_fil(i, j, k) = WTxyz7(i, j, k)
            enddo
        enddo
    enddo

    return
endsubroutine DYNfil_3D
